/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */


/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Or try here: http://www.fsf.org/copyleft/gpl.html
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <pwd.h>
#include <ctype.h>
#include <signal.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>


#include "lock.h"


int main(int argc, char **argv) 
{
    int i;
    for( i = 0; i < argc; i++ )
    {
	if ( ( strcmp( argv[ i ], "--help" ) == 0 ) ||
	     ( strcmp( argv[ i ], "-h" ) == 0 ) )
	{
	    printf ("\nlock - A simple console locker.\n\n");
	    printf ("usage: lock [<options>]\n" );
	    printf (" --help\t\t\tDisplays this message.\n");
	    printf (" --version\t\tDisplays the version.\n");
	    printf ("\nReport bugs to : Steve Kemp <skx@tardis.ed.ac.uk>\n");
	    exit( 0 );
	}
	if ( ( strcmp( argv[ i ], "--version" ) == 0 ) ||
	     ( strcmp( argv[ i ], "-v" ) == 0 ) )
	{
	    printf( "lock version %s - compiled on %s.\n",
		    VERSION, __DATE__ );
	    exit( 0 );
	}	
    }


    /*
     * Ignore all attempts at interruption.
     */
    signal(SIGINT , SIG_IGN);
    signal(SIGHUP , SIG_IGN);
    signal(SIGTERM, SIG_IGN);
    signal(SIGQUIT, SIG_IGN);
    signal(SIGTSTP, SIG_IGN);
    signal(SIGUSR1, SIG_IGN);
    signal(SIGUSR2, SIG_IGN);
 
    lockLoop();

    /* Clear up the screen after unlocking. */
    printf("%c[2J", ESCAPE_CODE);
    printf("%c[1;1f", ESCAPE_CODE);

    return 0;
}



/**
 *  Run an endless loop waiting for return presses, then try
 * to authenticate the user.
 * 
 */
void lockLoop(void) 
{
    struct passwd *pw = NULL;
    short error       = 1;

    /* Get the current users home directory */
    pw = getpwuid(getuid());
    if (!pw) 
    { 
	fprintf(stderr, "You don't exist!\n"); 
	exit(1); 
    }


    /* Loop. */
    while (error) 
    {
	updateDisplay();

	error = authenticate(pw->pw_name);

	if (error) 
	{
	    /* Beep */
	    fputc('\007',stderr);
	    fputc('\a',stderr);
	}
    }
}



/**
 * Attempt to authenticate the user.
 */
short authenticate(char *username) 
{
    short error;
    pam_handle_t *pamh = NULL;


    /* PAM conversation struct */
    static struct pam_conv conv = { misc_conv, NULL };
    
    /* start PAM session for 'lock'.          *
     * the session name is what PAM looks for *
     * in /etc/pam.d/ or /etc/pam.config      */
    error = pam_start("lock", username, &conv, &pamh);

    /* authenticate user */
    if (!error) 
    {
	error = pam_authenticate(pamh, 0);
    }

    /* close PAM session */
    if (pam_end(pamh,error) != PAM_SUCCESS) 
    {
	pamh = NULL;
	fprintf(stderr,"lock: ** failed to release PAM authenticator\n");
	error = 1;
    }

    return error;
}


/**
 * Clear the screen; display the timestamp.
 */
void updateDisplay(void) 
{
    time_t now;
    char time_now[100];
    memset( time_now, '\0', sizeof( time_now) );

    now = time( NULL );
    strftime( time_now, (sizeof( time_now) -1), 
              "%H:%M:%S %d %B %Y ", localtime( &now ) );

    printf("%c[2J", ESCAPE_CODE);    /* clear character screen */
    printf("%c[1;1f", ESCAPE_CODE);  /* place cursor at the top left. */
   
    printf("\nCurrent time: %s", time_now );
    printf("\n\n -- Press [Enter] to unlock --\n\n" );
   
    getchar();
   
}
